import { autoUpdate, computePosition, offset, flip, shift, arrow } from '@floating-ui/dom';
import jump from 'jump.js';

function createElement(tagName, props = {}, children = []) {
    const el = document.createElement(tagName);
    for (const [key, value] of Object.entries(props)) {
        if (value != null) {
            el.setAttribute(key, String(value));
        }
    }
    children.forEach((child) => {
        if (typeof child === "string") {
            el.appendChild(document.createTextNode(child));
        }
        else {
            el.appendChild(child);
        }
    });
    return el;
}
function getStaticSide(placement) {
    const sides = {
        top: "bottom",
        right: "left",
        bottom: "top",
        left: "right",
    };
    return sides[placement.split("-")[0]];
}
function isElementInViewport(element) {
    const rect = element.getBoundingClientRect();
    const windowHeight = window.innerHeight || document.documentElement.clientHeight;
    const windowWidth = window.innerWidth || document.documentElement.clientWidth;
    const verticallyVisible = (rect.top >= 0 && rect.bottom <= windowHeight);
    const horizontallyVisible = (rect.left >= 0 && rect.right <= windowWidth);
    return verticallyVisible && horizontallyVisible;
}
function getHighestZIndex() {
    let highest = 2000;
    const selectors = [
        'modal',
        'dialog',
        'drawer',
        'popup',
        'tooltip',
        'dropdown',
        'overlay',
        'mask',
    ].map(name => `[class*="${name}"]`).join(',');
    const elements = document.querySelectorAll(`
    ${selectors},
    [style*="z-index"],
    [style*="position: fixed"],
    [style*="position: absolute"]
  `);
    for (const element of elements) {
        const style = window.getComputedStyle(element);
        const position = style.position;
        if (position === 'fixed' || position === 'absolute') {
            const zIndex = style.zIndex;
            const zIndexNum = !isNaN(Number(zIndex)) ? Number(zIndex) : 0;
            highest = Math.max(highest, zIndexNum);
        }
    }
    return highest + 1;
}

class Tour {
    constructor(options) {
        var _a;
        Object.defineProperty(this, "targetElement", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: undefined
        });
        Object.defineProperty(this, "tooltipElement", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: undefined
        });
        Object.defineProperty(this, "arrowElement", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: undefined
        });
        Object.defineProperty(this, "positionCleanup", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: undefined
        });
        Object.defineProperty(this, "currentStep", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 0
        });
        Object.defineProperty(this, "steps", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        Object.defineProperty(this, "instanceConfig", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: undefined
        });
        const normalizedOptions = Array.isArray(options)
            ? { steps: options }
            : options;
        if (!((_a = normalizedOptions === null || normalizedOptions === undefined ? undefined : normalizedOptions.steps) === null || _a === undefined ? undefined : _a.length)) {
            return;
        }
        this.steps = normalizedOptions.steps;
        this.instanceConfig = normalizedOptions.config;
        this.createTooltipElements();
        if (this.getConfig().autoStart) {
            this.run();
        }
    }
    getConfig() {
        return Object.assign(Object.assign({}, Tour.config), this.instanceConfig);
    }
    checkTargetElement() {
        const step = this.steps[this.currentStep];
        return document.querySelector(step.target);
    }
    initStep() {
        var _a, _b;
        (_a = this.positionCleanup) === null || _a === undefined ? undefined : _a.call(this);
        const targetElement = this.checkTargetElement();
        if (!targetElement)
            return;
        (_b = this.targetElement) === null || _b === undefined ? undefined : _b.classList.remove("_tourjs-target--highlighted");
        this.targetElement = targetElement;
        if (this.tooltipElement) {
            this.tooltipElement.setAttribute('data-show', 'false');
        }
        if (!isElementInViewport(targetElement)) {
            jump(this.targetElement, {
                offset: -10,
                duration: 300,
                callback: () => this.showTooltip()
            });
        }
        else {
            setTimeout(() => {
                this.showTooltip();
            }, 300);
        }
    }
    showTooltip() {
        if (!this.targetElement || !this.tooltipElement)
            return;
        this.targetElement.classList.add("_tourjs-target--highlighted");
        this.updateTooltipContent();
        requestAnimationFrame(() => {
            if (this.tooltipElement && this.targetElement) {
                this.positionCleanup = autoUpdate(this.targetElement, this.tooltipElement, this.updatePosition());
                this.tooltipElement.setAttribute('data-show', 'true');
            }
        });
    }
    cleanup() {
        var _a, _b;
        if (this.tooltipElement) {
            this.tooltipElement.setAttribute('data-show', 'false');
            setTimeout(() => {
                var _a;
                (_a = this.tooltipElement) === null || _a === undefined ? undefined : _a.remove();
                this.tooltipElement = undefined;
                this.arrowElement = undefined;
            }, 300);
        }
        (_a = this.positionCleanup) === null || _a === undefined ? undefined : _a.call(this);
        (_b = this.targetElement) === null || _b === undefined ? undefined : _b.classList.remove("_tourjs-target--highlighted");
        this.targetElement = undefined;
        this.currentStep = 0;
    }
    updatePosition() {
        return () => {
            if (!this.targetElement || !this.tooltipElement || !this.arrowElement)
                return;
            computePosition(this.targetElement, this.tooltipElement, {
                placement: "bottom",
                middleware: [
                    offset(12),
                    flip(),
                    shift({ padding: 5 }),
                    arrow({ element: this.arrowElement }),
                ],
            }).then(({ x, y, placement, middlewareData }) => {
                Object.assign(this.tooltipElement.style, {
                    left: `${x}px`,
                    top: `${y}px`,
                });
                const { x: arrowX, y: arrowY } = middlewareData.arrow || {};
                const staticSide = getStaticSide(placement);
                Object.assign(this.arrowElement.style, {
                    left: arrowX != null ? `${arrowX}px` : "",
                    top: arrowY != null ? `${arrowY}px` : "",
                    right: "",
                    bottom: "",
                    [staticSide]: "-4px",
                });
            });
        };
    }
    createTooltipElements() {
        var _a;
        if (this.tooltipElement)
            return;
        this.arrowElement = createElement("div", { class: "tourjs-step-arrow" });
        this.tooltipElement = createElement("div", { class: "tourjs-step" }, [this.arrowElement]);
        const baseZIndex = this.getConfig().zIndex;
        const finalZIndex = baseZIndex === 'auto' ? getHighestZIndex() : baseZIndex;
        this.tooltipElement.style.setProperty('z-index', String(finalZIndex));
        (_a = this.targetElement) === null || _a === undefined ? undefined : _a.style.setProperty('z-index', String(finalZIndex - 1));
        const content = createElement("div", { class: "tourjs-step-content" });
        const buttons = this.createButtons();
        const btnGroup = createElement("div", { class: "tourjs-step-btn-group" }, buttons);
        this.tooltipElement.appendChild(content);
        this.tooltipElement.appendChild(btnGroup);
        document.body.appendChild(this.tooltipElement);
    }
    updateTooltipContent() {
        if (!this.tooltipElement)
            return;
        const content = this.tooltipElement.querySelector('.tourjs-step-content');
        if (content) {
            content.textContent = this.steps[this.currentStep].content;
        }
        const btnGroup = this.tooltipElement.querySelector('.tourjs-step-btn-group');
        if (btnGroup) {
            btnGroup.replaceWith(createElement("div", { class: "tourjs-step-btn-group" }, this.createButtons()));
        }
    }
    createButtons() {
        const btnPrevious = createElement("button", {
            class: `tourjs-step-btn ${this.isFirst() ? 'tourjs-step-btn-disabled' : ''}`,
            disabled: this.isFirst() ? "disabled" : undefined
        }, ["上一步"]);
        const btnNext = createElement("button", {
            class: 'tourjs-step-btn tourjs-step-btn-primary',
        }, [this.isLast() ? "完成" : "下一步"]);
        const buttons = [btnPrevious, btnNext];
        if (!this.isLast()) {
            const btnSkip = createElement("button", {
                class: "tourjs-step-btn tourjs-step-btn-skip"
            }, ["跳过"]);
            btnSkip.addEventListener('click', () => this.skip());
            buttons.push(btnSkip);
        }
        btnPrevious.addEventListener('click', () => {
            if (this.currentStep > 0) {
                this.previous();
            }
        });
        btnNext.addEventListener('click', () => {
            if (this.currentStep === this.steps.length - 1) {
                this.cleanup();
            }
            else {
                this.next();
            }
        });
        return buttons;
    }
    next() {
        if (this.currentStep < this.steps.length - 1) {
            this.currentStep++;
            const targetElement = this.checkTargetElement();
            if (!targetElement) {
                if (this.getConfig().autoSkipInvalid && this.currentStep < this.steps.length - 1) {
                    return this.next();
                }
                this.cleanup();
                throw new Error(`目标元素 ${this.steps[this.currentStep].target} 未找到`);
            }
            this.initStep();
        }
        else {
            this.skip();
        }
    }
    previous() {
        if (this.currentStep > 0) {
            this.currentStep--;
            const targetElement = this.checkTargetElement();
            if (!targetElement) {
                if (this.getConfig().autoSkipInvalid && this.currentStep > 0) {
                    return this.previous();
                }
                this.cleanup();
                throw new Error(`目标元素 ${this.steps[this.currentStep].target} 未找到`);
            }
            this.initStep();
        }
    }
    skip() {
        this.cleanup();
    }
    static configure(options) {
        Object.assign(Tour.config, options);
    }
    goTo(index) {
        if (index < 0 || index >= this.steps.length) {
            throw new Error(`步骤索引 ${index} 不合法，有效范围: 0-${this.steps.length - 1}`);
        }
        this.currentStep = index;
        const targetElement = this.checkTargetElement();
        if (!targetElement) {
            this.cleanup();
            throw new Error(`目标元素 ${this.steps[this.currentStep].target} 未找到`);
        }
        this.initStep();
    }
    getCurrentStep() {
        return this.currentStep;
    }
    getTotalSteps() {
        return this.steps.length;
    }
    isFirst() {
        return this.currentStep === 0;
    }
    isLast() {
        return this.currentStep === this.steps.length - 1;
    }
    run() {
        this.currentStep = 0;
        this.initStep();
    }
}
Object.defineProperty(Tour, "config", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: {
        autoSkipInvalid: true,
        autoStart: true,
        zIndex: 9999
    }
});

export { Tour, createElement, getStaticSide, isElementInViewport };
